package edu.georgetown.gucs.fingerprinter;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.logging.Level;
import java.util.logging.Logger;

import edu.georgetown.gucs.utility.SdTextException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import edu.georgetown.gucs.dictionary.Dictionary;
import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Global;
import org.slf4j.LoggerFactory;

/**
 * Produces a {@linkplain Dictionary} in XML format from a given {@linkplain Fingerprinter} digest containing a
 * dictionary.
 *
 * @author Lindsay Neubauer
 */
public class ExtractFingerprintDictionary {

    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(ExtractFingerprintDictionary.class);
    /*
     * Produces an XML dictionary object from a fingerprint digest. If the <code>dictionary_file</code> is null, the
     * dictionary contained in the fingerprinter digest is used.
     *
     * @param input_file
     *          the string name of the XML file containing the fingerprint digest
     * @param dictionary_file
     *          the string name of the XML file to write the dictionary to, if null the name of the dictionary in the
     *          fingerprint digest is used
     */
    public static void extractDictionaryXML(String input_file, String dictionary_file) throws SdTextException {
        try {
            File xml = new File(input_file);
            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(xml);

            Element fingerprinter = doc.getRootElement();
            String xmlFilename = (dictionary_file == null || dictionary_file.length() < 1) ? fingerprinter.getChild(
                    "dictionary").getAttributeValue("name") : (new File(dictionary_file)).getAbsolutePath();
            Document dictionary_xml = new Document((Element) fingerprinter.getChild("dictionary").clone());

            FileOutputStream digest = new FileOutputStream(xmlFilename);
            Format form = Format.getPrettyFormat();
            form.setEncoding(Global.fileEncoding);
            XMLOutputter out = new XMLOutputter(form);
            out.output(dictionary_xml, digest);
            System.out.println("Dictionary in " + xml.getAbsolutePath() + " written to " + xmlFilename);
            System.out.println("ExtractFingerprintDictionary - extractDictionaryXML - Writing dictionary to "
                    + xmlFilename);
            digest.close();
        } catch (JDOMException | IOException e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Produces an XML dictionary object from a fingerprint digest. If the <code>dictionary_file</code> is null,
     * the program outputs to the console.
     *
     * @param input_file
     *          the string name of the XML file containing the fingerprint digest
     */
    public static void extractDictionaryXML(String input_file) throws SdTextException {
        try {
            File xml = new File(input_file);
            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(xml);

            // Element fingerprinter = doc.getRootElement();
            //Document dictionary_xml = new Document((Element) fingerprinter.getChild("dictionary").clone());
            Format form = Format.getPrettyFormat();
            form.setEncoding(Global.fileEncoding);
            XMLOutputter printer = new XMLOutputter(form);
           logger.info(doc.toString());
            printer.output(doc, System.out);


            logger.info( "Writing Dictionary to Console");

        } catch (JDOMException | IOException e) {
            ExceptionHandler.handleException(e);
        }
    }


}