package edu.georgetown.gucs.tokenizers;

import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.Vector;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.Token;

/**
 * Uses English stemming to change tokens into their root form
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 *
 */
public class PorterTokenizer extends Tokenizer {

    private static final long serialVersionUID = 1555744102819134473L;

    /**
     * Changes English language tokens into their root form
     */
    @Override
    public List<Token> tokenize(List<Token> tokens) throws SdTextException {

        List<Token> tokenVector = Collections.synchronizedList(new Vector<Token>());
        try {
            synchronized (tokenVector) {
                Stemmer porterStemmer = new Stemmer();
                for (Token token : tokens) {
                    String lc_token = token.name().toLowerCase(Locale.ENGLISH);
                    for (int i = 0; i < lc_token.length(); i++) {
                        porterStemmer.add(lc_token.charAt(i));
                    }
                    porterStemmer.stem();
                    tokenVector.add(new Token(porterStemmer.toString(), token.start_byte(), token.end_byte()));
                }
                return tokenVector;
            }
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return tokenVector;
    }
}
