package edu.georgetown.gucs.utility;

import java.io.Serializable;
import java.util.Comparator;

/**
 * Provides the token parameters, including the name and byte information
 *
 * @author Lindsay Neubauer
 */
public class Token implements Serializable, Comparable {

    private static final long serialVersionUID = 1585236721004515366L;
    private final String name;
    private final int start_byte;
    private final int end_byte;

    /**
     * Constructor that sets all attributes of a token
     *
     * @param name       the string name of the token
     * @param start_byte the <code>int</code> value of the first byte of this token; set to -1 if no information is available
     * @param end_byte   the <code>int</code> value of the last byte of this token; set to -1 if no information is available
     */
    public Token(String name, int start_byte, int end_byte) {
        this.name = name;
        this.start_byte = start_byte;
        this.end_byte = end_byte;
    }

    /**
     * Constructor that sets all attributes of a token
     *
     * @param name the string name of the token
     */
    public Token(String name) {
        this.name = name;
        this.start_byte = 0;
        this.end_byte = 0;
    }

    /**
     * Provides the token name
     *
     * @return the string name of the token
     */
    public String name() {
        return this.name;
    }

    /**
     * Provides the starting byte location of this token
     *
     * @return the <code>int</code> value of the first byte of this token
     */
    public int start_byte() {
        return this.start_byte;
    }

    /**
     * Provides the ending byte location of this token
     *
     * @return the <code>int</code> value of the last byte of this token
     */
    public int end_byte() {
        return this.end_byte;
    }

    @Override
    public String toString() {
        return this.name;// + " " + this.start_byte + " " + this.end_byte;
    }

    /**
     * Override the hashcode method so it will sort on name
     */
    @Override
    public int hashCode() {
        return this.name.hashCode();
    }

    /**
     * Override the equals so that comparison is based on the name
     */

    @Override
    public boolean equals(Object tokenObject) {
        if (tokenObject == null) {
            return false;
        }
        if (tokenObject == this) {
            return true;
        }
        if (!(tokenObject instanceof Token)) {
            return false;
        }
        Token token = (Token) tokenObject;
        return this.name().equals(token.name());
    }


    @Override
    public int compareTo(Object o) {
        return name.compareTo(o.toString());
    }
}
